@@ -11,6 +11,53 @@ from copy import deepcopy
 from typing import List
 
 
+NOTENUM_TO_NOTENAME_DICT = {
+    12: 'C0', 13: 'Db0', 14: 'D0', 15: 'Eb0', 16: 'E0', 17: 'F0',
+    18: 'Gb0', 19: 'G0', 20: 'Ab0', 21: 'A0', 22: 'Bb0', 23: 'B0',
+     24: 'C1', 25: 'Db1', 26: 'D1', 27: 'Eb1', 28: 'E1', 29: 'F1',
+     30: 'Gb1', 31: 'G1', 32: 'Ab1', 33: 'A1', 34: 'Bb1', 35: 'B1',
+     36: 'C2', 37: 'Db2', 38: 'D2', 39: 'Eb2', 40: 'E2', 41: 'F2',
+     42: 'Gb2', 43: 'G2', 44: 'Ab2', 45: 'A2', 46: 'Bb2', 47: 'B2',
+     48: 'C3', 49: 'Db3', 50: 'D3', 51: 'Eb3', 52: 'E3', 53: 'F3',
+     54: 'Gb3', 55: 'G3', 56: 'Ab3', 57: 'A3', 58: 'Bb3', 59: 'B3',
+     60: 'C4', 61: 'Db4', 62: 'D4', 63: 'Eb4', 64: 'E4', 65: 'F4',
+     66: 'Gb4', 67: 'G4', 68: 'Ab4', 69: 'A4', 70: 'Bb4', 71: 'B4',
+     72: 'C5', 73: 'Db5', 74: 'D5', 75: 'Eb5', 76: 'E5', 77: 'F5',
+     78: 'Gb5', 79: 'G5', 80: 'Ab5', 81: 'A5', 82: 'Bb5', 83: 'B5',
+     84: 'C6', 85: 'Db6', 86: 'D6', 87: 'Eb6', 88: 'E6', 89: 'F6',
+     90: 'Gb6', 91: 'G6', 92: 'Ab6', 93: 'A6', 94: 'Bb6', 95: 'B6',
+     96: 'C7', 97: 'Db7', 98: 'D7', 99: 'Eb7', 100: 'E7', 101: 'F7',
+     102: 'Gb7', 103: 'G7', 104: 'Ab7', 105: 'A7', 106: 'Bb7', 107: 'B7',
+     108: 'C8', 109: 'Db8', 110: 'D8', 111: 'Eb8', 112: 'E8', 113: 'F8',
+     114: 'Gb8', 115: 'G8', 116: 'Ab8', 117: 'A8', 118: 'Bb8', 119: 'B8',
+     120: 'C9', 121: 'Db9', 122: 'D9', 123: 'Eb9', 124: 'E9', 125: 'F9',
+     126: 'Gb9', 127: 'G9'
+}
+
+# TODO: シャープに対応する。
+NOTENAME_TO_NOTENUM_DICT = {
+    'C0': 12, 'C#0': 13, 'Db0': 13, 'D0': 14, 'D#0': 15, 'Eb0': 15, 'E0': 16, 'F0': 17, 'F#0': 18,
+    'Gb0': 18, 'G0': 19, 'G#0': 20, 'Ab0': 20, 'A0': 21, 'A#0': 22, 'Bb0': 22, 'B0': 23,
+    'C1': 24, 'C#1': 25, 'Db1': 25, 'D1': 26, 'D#1': 27, 'Eb1': 27, 'E1': 28, 'F1': 29, 'F#1': 30,
+    'Gb1': 30, 'G1': 31, 'Ab1': 32, 'A1': 33, 'Bb1': 34, 'B1': 35,
+    'C2': 36, 'Db2': 37, 'D2': 38, 'Eb2': 39, 'E2': 40, 'F2': 41,
+    'Gb2': 42, 'G2': 43, 'Ab2': 44, 'A2': 45, 'Bb2': 46, 'B2': 47,
+    'C3': 48, 'Db3': 49, 'D3': 50, 'Eb3': 51, 'E3': 52, 'F3': 53,
+    'Gb3': 54, 'G3': 55, 'Ab3': 56, 'A3': 57, 'Bb3': 58, 'B3': 59,
+    'C4': 60, 'C#4': 61, 'Db4': 61, 'D4': 62, 'D#4': 63, 'Eb4': 63, 'E4': 64, 'F4': 65, 'F#4': 66,
+    'Gb4': 66, 'G4': 67, 'Ab4': 68, 'A4': 69, 'Bb4': 70, 'B4': 71,
+    'C5': 72, 'Db5': 73, 'D5': 74, 'Eb5': 75, 'E5': 76, 'F5': 77,
+    'Gb5': 78, 'G5': 79, 'Ab5': 80, 'A5': 81, 'Bb5': 82, 'B5': 83,
+    'C6': 84, 'Db6': 85, 'D6': 86, 'Eb6': 87, 'E6': 88, 'F6': 89,
+    'Gb6': 90, 'G6': 91, 'Ab6': 92, 'A6': 93, 'Bb6': 94, 'B6': 95,
+    'C7': 96, 'Db7': 97, 'D7': 98, 'Eb7': 99, 'E7': 100, 'F7': 101,
+    'Gb7': 102, 'G7': 103, 'Ab7': 104, 'A7': 105, 'Bb7': 106, 'B7': 107,
+    'C8': 108, 'Db8': 109, 'D8': 110, 'Eb8': 111, 'E8': 112, 'F8': 113,
+    'Gb8': 114, 'G8': 115, 'Ab8': 116, 'A8': 117, 'Bb8': 118, 'B8': 119,
+    'C9': 120, 'Db9': 121, 'D9': 122, 'Eb9': 123, 'E9': 124, 'F9': 125,
+    'Gb9': 126, 'G9': 127
+}
+
 def notenum_as_abc(notenum) -> str:
     """
     音階番号をABC表記に変更する(C1=24, C4=)
@@ -39,7 +86,7 @@ def load(path: str, encoding: str = 'shift-jis'):
     USTを読み取り
     """
     new_ust = Ust()
-    new_ust.load(path, encoding=encoding)
+    new_ust.load(path.strip('"'), encoding=encoding)
     return new_ust
 
 
@@ -52,17 +99,17 @@ class Ust:
         super().__init__()
         # ノート(クラスオブジェクト)からなるリスト
         self.version = None   # [#VERSION]
-        self.setting = None   # [#SETTING]
-        self.notes: List[Note] = []       # [#1234], [#INSERT], [#DELETE]
-        self.trackend = None  # [#TRACKEND]
-        self.next_note = None      # [#NEXT]
-        self.previous_note = None      # [#PREV]
+        self.setting = Note(tag='[#SETTING]')  # [#SETTING]
+        self.notes: List[Note] = []  # [#1234], [#INSERT], [#DELETE]
+        self.trackend = Note(tag='[#TRACKEND]')  # [#TRACKEND]
+        self.next_note = None  # [#NEXT]
+        self.previous_note = None  # [#PREV]
 
     def __str__(self):
         # self.notesが増減するので複製したものを扱う
         duplicated_self = deepcopy(self)
         # 特殊ノートを処理
-        if self.setting is not None:
+        if len(self.setting) >= 2:
             duplicated_self.notes.insert(0, self.setting)
         if self.previous_note is not None:
             duplicated_self.notes.insert(0, self.previous_note)
@@ -129,7 +176,7 @@ class Ust:
         self.reload_tempo()
         return self
 
-    @ property
+    @property
     def tempo(self) -> float:
         """
         USTのグローバルテンポ
@@ -141,7 +188,7 @@ class Ust:
             self.setting['Tempo'] = self.notes[0].tempo
         return self.setting['Tempo']
 
-    @ tempo.setter
+    @tempo.setter
     def tempo(self, tempo):
         self.setting.tempo = tempo
         # self.notes[0].tempo = tempo
@@ -242,6 +289,8 @@ class Ust:
         duplicated_self.notes = [
             note for note in duplicated_self.notes if note.tag != '[#DELETE]'
         ]
+        # テンポを整理する
+        duplicated_self.reload_tempo()
         # ノート番号を振りなおす
         duplicated_self.reload_tag_number()
         # 文字列にする
@@ -292,7 +341,7 @@ class Note(UserDict):
     @length_ms.setter
     def length_ms(self, x):
         """ノート長を上書き[ms]"""
-        self['Length'] = x * self.tempo // 125
+        self['Length'] = str(round(x * self.tempo / 125))
 
     @property
     def lyric(self):
@@ -316,6 +365,17 @@ class Note(UserDict):
     def notenum(self, x):
         self['NoteNum'] = str(x)
 
+    @property
+    def notename(self):
+        """
+        音名（C4などの表記）
+        """
+        return NOTENUM_TO_NOTENAME_DICT[self.notenum]
+
+    @notename.setter
+    def notename(self, notename):
+        self.notenum = NOTENAME_TO_NOTENUM_DICT[str(notename)]
+
     @property
     def tempo(self) -> float:
         """
@@ -328,9 +388,9 @@ class Note(UserDict):
         self['Tempo'] = str(x)
 
     @property
-    def pbs(self):
+    def pbs(self) -> list:
         """
-        PBS (mode2ピッチ開始位置[ms]) を取得
+        PBS (mode2ピッチ開始位置[ms])
         例) PBS=-104;20.0
         """
         # 辞書には文字列で登録してある
@@ -342,10 +402,6 @@ class Note(UserDict):
 
     @pbs.setter
     def pbs(self, list_pbs):
-        """
-        PBS (mode2ピッチ開始位置[ms]) を登録
-        例) PBS=-104;20.0
-        """
         s1 = f'{int(list_pbs[0])};'
         s2 = ','.join(map(str, list_pbs[1:]))
 
@@ -353,7 +409,7 @@ class Note(UserDict):
         self['PBS'] = str_pbs
 
     @property
-    def pbw(self):
+    def pbw(self) -> list:
         """
         PBW (mode2ピッチ点の間隔[ms]) を取得
         例) PBW=77,163
@@ -367,16 +423,12 @@ class Note(UserDict):
 
     @pbw.setter
     def pbw(self, list_pbw):
-        """
-        PBW (mode2ピッチ点の間隔[ms]) を登録
-        例) PBW=77,163
-        """
         # リストを整数の文字列に変換
         str_pbw = ','.join(list(map(str, map(int, list_pbw))))
         self['PBW'] = str_pbw
 
     @property
-    def pby(self):
+    def pby(self) -> list:
         """
         PBY (mode2ピッチ点の高さ) を取得
         例) PBY=-10.6,0.0
@@ -390,16 +442,12 @@ class Note(UserDict):
 
     @pby.setter
     def pby(self, list_pby):
-        """
-        PBY (mode2ピッチ点の高さ) を登録
-        例) PBY=-10.6,0.0
-        """
         # リストを小数の文字列に変換
         str_pby = ','.join(list(map(str, map(float, list_pby))))
         self['PBY'] = str_pby
 
     @property
-    def pbm(self):
+    def pbm(self) -> list:
         """
         PBM (mode2ピッチ点の形状) を取得
         例) PBY=-10.6,0.0
@@ -413,10 +461,6 @@ class Note(UserDict):
 
     @pbm.setter
     def pbm(self, list_pbm):
-        """
-        PBM (mode2ピッチ点の形状) を登録
-        例) PBM=,r,j,s
-        """
         # リストを文字列に変換
         str_pbm = ','.join(list_pbm)
         self['PBM'] = str_pbm
@@ -432,6 +476,29 @@ class Note(UserDict):
     def velocity(self, x: int):
         self['Velocity'] = int(x)
 
+    @property
+    def flags(self) -> str:
+        """
+        フラグ
+        """
+        return self.get('Flags', '')
+
+    @flags.setter
+    def flags(self, flags: str):
+        self['Flags'] = str(flags)
+
+    @property
+    def label(self) -> str:
+        """
+        ノートのラベル
+        UTAUエディタのピアノロール画面で、ローカルテンポの上に表示されるやつ
+        """
+        return self.get('Label', '')
+
+    @label.setter
+    def label(self, label: str):
+        self['Label'] = str(label)
+
     # ここからデータ操作系-----------------------------------------------------
     def get_by_key(self, key):
         """
